# -*- python -*-
# $RCSfile: activityViewer.py,v $
# $Revision: 1.14 $
# $Author: langer $
# $Date: 2004/07/14 16:26:37 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 



from SWIG.common.IO import progressbar
from SWIG.common.switchboard import switchboard
from common import debug
from common import mainthread
from common import thread_enable
from common.IO import gfxmanager
from common.IO import progressbar_delay
from common.IO.GUI import gfxmenu
from common.IO.GUI import quit
from common.IO.GUI import subWindow
from common.IO.oofmenu import OOFMenuItem
from gtk import *
import gtk
import string
import sys
import time
##from common.IO.GUI import progress_enabled



from common.IO.threadmanager import threadManager
from common.IO import threadmanagermenu
activityViewer = None

## activityViewerCounter counts how many times the viewer has been
## opened. If it's been opened once, it's not opened automatically
## anymore.
activityViewerCounter = 0

def openActivityViewer():
    debug.mainthreadTest()

    global activityViewer
    global activityViewerCounter
    if activityViewer is None:
        activityViewer = ActivityViewer()
##        progress_enabled.set(1)
    activityViewer.raise_window()
    activityViewerCounter +=1

switchboard.requestCallback("new activity viewer", openActivityViewer)


def sensitize_thread():
    if activityViewer is not None:
        activityViewer.sensitizeButtons()
def sensitize():
    mainthread.runBlock(sensitize_thread)
    
        
class ActivityViewer(subWindow.SubWindow):
    ## All the progress bars should update themselves once every 
    ## specified number of miliseconds. 
    ## should be a scrolled clist of progress bars
    
    def __init__(self):
        debug.mainthreadTest()
        self.sbcallbacks = [
            switchboard.requestCallback('activity viewer close', self.close),
            switchboard.requestCallback("dismiss gtkbar", self.dismissgtkbar)
##            ,
##            switchboard.requestCallback('sensitize activity viewer',self.sensitizeButtons)
            ]            
        
        self.listofgtkbars = []
        
        self.makeMenus()  # Sets self.menu.
        # SubWindow init sets self.gtk and self.mainbox.
        subWindow.SubWindow.__init__(
            self, title="OOF2 Activity Viewer",
            menu=self.menu)

        self.gtk.connect('destroy', self.closeCB)
        self.gtk.set_usize(400, 300)
        self.gtk.set_title("OOF2 Activity Viewer")
        self.gtk.set_policy(1, 1, 0)
        
        # Area at the top containing the editor widget for the line
        vbox = GtkVBox()
        self.mainbox.add(vbox)
        self.control_area = GtkHBox()       # editor widget goes in here
        vbox.pack_start(self.control_area, gtk.FALSE, gtk.FALSE, 0)

        ## create enable-threading button
        self.dismissall = gtk.GtkButton("Dismiss All")
        self.dismissall.connect("clicked", self.dismissAll)
        self.control_area.add(self.dismissall)
        self.dismissall.show()

        ## create stop-all-threads button
        self.stopall = gtk.GtkButton("Stop All")
        self.stopall.connect("clicked", self.stopAll)
        self.control_area.add(self.stopall)
        self.stopall.show()

                
        # Create a scrolled window to pack the vbox into
        self.scrolled_window = gtk.GtkScrolledWindow()
        self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        vbox.pack_start(self.scrolled_window, gtk.TRUE, gtk.TRUE, 0)
        self.scrolled_window.show()
        
        ## Create VBox where the progressbars can live happily ever after
        self.bars_play_area = GtkVBox(gtk.TRUE,0)
        self.scrolled_window.add_with_viewport(self.bars_play_area)
        self.bars_play_area.show()
        
        ## Buttons become sensitized
        self.sensitizeButtons()
        ## show window

        self.initialize() ## add progress bars to Window
        self.gtk.show_all()
        self.sensitizeButtons()
        
    def initialize(self):
        ## This call should query existing progressbars in ThreadMonitor
        ## and reconstruct the Window.
        debug.mainthreadTest()
        if not thread_enable.enabled():
            return
        
        for pbar in threadManager.listofprogressbars:
            a_newgbar = progressbar.set_ProgressBarRepresentation(pbar)
            self.bars_play_area.pack_start(a_newgbar.get_Gtkbar(),
                                           gtk.FALSE, gtk.FALSE, 0)
            a_newgbar.show()
            self.listofgtkbars.append(a_newgbar)
        
    

    def sensitizeButtons(self):
        debug.mainthreadTest()
        threads_in_progress = threadManager.NumThreadsInProgress()
        threads_done = threadManager.NumThreadsDone()
        
        if threads_in_progress >0:
            self.stopall.set_sensitive(1)
        else:
            self.stopall.set_sensitive(0)
            
        if threads_done >0:
            self.dismissall.set_sensitive(1)
        else:
            self.dismissall.set_sensitive(0)

        if not thread_enable.enabled():
            self.stopall.set_sensitive(0)
            self.dismissall.set_sensitive(0)
        
            
    def dismissAll(self, button = None):
        ## print "dismiss command issued"
        itemsremoved = []
        for bar in self.listofgtkbars:
            if bar.get_success() !=0 :
                itemsremoved.append(bar)
        for rbar in itemsremoved:
            switchboard.notify("remove progressbar", rbar.progressbar)
            rbar.schedule_gtk_removal()
        self.sensitizeButtons()
        itemsremoved = []
        
    def stopAll (self, button):
        threadManager.stopAll(button)
        self.sensitizeButtons()

    def dismissgtkbar(self,thisbar):
        ## print "removing gtkbar from gtklist"
        self.listofgtkbars.remove(thisbar)
        self.sensitizeButtons()
    
    def makeMenus(self): ## Adds quit Button
        self.menu = threadmanagermenu.threadmanagermenu
        self.menu.File.addItem(OOFMenuItem('Quit',
                                           callback=quit.quit,
                                           no_log = 1,
                                           gui_only = 1,
                                           accel='q'))
        

    
    #################################
        
    
    def closeCB(self, *args):
        global activityViewer
        if activityViewer:
            threadmanagermenu.threadmanagermenu.File.Close()
            
        
    def close(self, *args):
        debug.mainthreadTest()
        global activityViewer
        if self.gtk:
            activityViewer = None
            self.gtk.destroy()
            self.gtk = None
        map(switchboard.removeCallback, self.sbcallbacks)
##        progress_enabled.set(0)


    def addGtkbar (self, pbar):
        debug.mainthreadTest()
        ## Adds a gtkprogressbar to the scrolled window.
        
        graphicbar = progressbar.set_ProgressBarRepresentation(pbar)
        self.bars_play_area.pack_start(graphicbar.get_Gtkbar(),
                                       gtk.FALSE, gtk.FALSE, 0)
        graphicbar.show()
        self.listofgtkbars.append(graphicbar)
        self.sensitizeButtons()

    
    #####################################################

def delayed_add_ProgressBar(pbar):
    if progressbar_delay.get_delay() <= progressbar_delay.minimum_time():
        ## print "adding pbar directly"
        add_ProgressBar(pbar)
    else:
        ## print "delaying pbar creation"
        gtk.timeout_add(progressbar_delay.get_delay(), add_ProgressBar, pbar)

def add_ProgressBar(pbar):
    if pbar.get_success():
        ## print "already done",pbar.get_success()
        switchboard.notify("remove progressbar", pbar)
        return gtk.FALSE
    if pbar.get_threadstate() == mainthread.mainthreadState:
        switchboard.notify("remove progressbar", pbar)
        return gtk.FALSE
    global activityViewerCounter
    global activityViewer
    if thread_enable.query() :
        if activityViewerCounter < 1:
            openActivityViewer()
            return gtk.FALSE
        if activityViewer is not None:
            activityViewer.addGtkbar(pbar)
            return gtk.FALSE
        
switchboard.requestCallback("new progressbar", delayed_add_ProgressBar)
## Switchboard registration guarantees that callback will occur
## in main thread.
