# -*- python -*-
# $RCSfile: datafile.py,v $
# $Revision: 1.16 $
# $Author: langer $
# $Date: 2004/08/08 02:27:38 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import debug
from common import enum
from common import utils
from common import version
from common.IO import mainmenu
from common.IO import menuparser
from common.IO import oofmenu
from common.IO import parameter
import string
import types

##############################

datafileversion = 1.0

##############################

# Data file formats

class DataFileFormat(enum.EnumClass(
    ('script', 'A fully functioning Python script.  Flexible and editable, but insecure.'),
    ('ascii', 'An ASCII file with Python-like syntax that will NOT be parsed by the Python interpreter.  Inflexible, editable, but secure.'),
    ('binary', 'A binary file. Inflexible and uneditable, but secure, compact and not subject to round-off error.')
    )): pass
utils.OOFdefine('DataFileFormat', DataFileFormat)

SCRIPT = DataFileFormat("script")
ASCII = DataFileFormat("ascii")
BINARY = DataFileFormat("binary")

##############################

def versionCB(menuitem, number, format):
    if format == BINARY:
        menuitem.parser.binaryMode()

versionCmd = oofmenu.OOFMenuItem(
    'FileVersion',
    callback=versionCB,
    params=[parameter.FloatParameter('number'),
            enum.EnumParameter('format', DataFileFormat)],
    help="Identify data file type.  Used internally in data files.")
    
mainmenu.OOF.LoadData.addItem(versionCmd)

class AsciiDataFile:
    def __init__(self, file, format):
        self.format = format
        self.file = file
        self.nargs = 0
        self.buffer = ""
    def startCmd(self, command):
        path = command.path()
        if self.format == ASCII:
            path = string.join(path.split('.')[2:], '.')
        self.buffer = path + "("
        self.nargs = 0
    def endCmd(self):
        self.file.write(self.buffer)
        self.file.write(")\n")
        self.buffer = ""
    def discardCmd(self):
        self.buffer = ""
        self.nargs = 0
    def argument(self, name, value):
        if self.nargs > 0:
            self.buffer += ", "
        self.buffer += "%s=%s" % (name, `value`)
        self.nargs += 1
    def comment(self, remark):
        self.file.write("# %s\n" % remark)
    def close(self):
        self.file.close()

def writeDataFile(filename, mode, format):
    if format == BINARY:
        mode += 'b'
    file = open(filename, mode)
    file.write("# OOF version %s\nFileVersion(number=%s, format=%s)\n"
               % (version.version, datafileversion, `format`))
    if format != BINARY:
        return AsciiDataFile(file, format)
    from common.IO import binarydata    # avoid import loop
    return binarydata.BinaryDataFile(file)

def readDataFile(filename, menu, progbar):
    source = menuparser.ProgFileInput(filename, progbar)
    parser = menuparser.MenuParser(source, menu)
    try:
        parser.run()
        if not source.error():
            progbar.set_success()
    except:
        if source.error():
            progbar.set_failure()
        raise
