# -*- python -*-
# $RCSfile: threadmanager.py,v $
# $Revision: 1.10 $
# $Author: langer $
# $Date: 2004/04/20 18:51:22 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import string
from SWIG.common.switchboard import switchboard
from common import debug
from common import guitop
from common import thread_enable
from common.IO import reporter


## ThreadMonitor coordinates the thread-enabled menu items of OOF.
## It administrates the existing running threaded processes,
## as well as those threads that have already finished and are waiting to be
## dismissed. It also sends instruction for automatically stoping them
## when the appropriate flag is set.

class ThreadManager: 
    def __init__(self):
        
        self.open_first_time = 0
        self.AutoDismiss = 0        
        self.listofprogressbars = []
        self.listofworkers = [] ## Keep list of workers in order to quit properly.
        ## At exit time, the ThreadManager loops over the running workers and forces
        ## them to stop and join. If this action is not enforced, an error is raised.
        
        self.sbcallbacks = [
            switchboard.requestCallback('remove progressbar', self.removeProgressBar),
            switchboard.requestCallback('remove worker', self.removeWorker)
            ]

        
    def close(self, *args):
        ## This member calls ThreadManagerGUI.close()
        switchboard.notify('activity viewer close')
        
    def WaitForAllThreads(self):
        for worker in self.listofworkers :
            worker.join()
            
    def quit(self, menuitem = None):
        ## print "stopping threads"
        self.stopAll()                  ## tell the threads politely to stop
        self.cancelAll()                ## tell the threads IMpolitely to stop
        ##self.WaitForAllThreads()
        map(switchboard.removeCallback, self.sbcallbacks)
        
        

    def disableAutoDismiss(self, menuitem, value):
        self.AutoDismiss = value

    def enableThreadingCB(self, menuitem, value):
        thread_enable.set(value)
        
        
    def removeProgressBar(self, pbarobj):
        ## print "removing text bar from list"
        self.listofprogressbars.remove(pbarobj) ## removes progress bar from list
                

    def stopAll(self, widget = None):
        for thisbar in self.listofprogressbars:
            if thisbar.get_success() == 0 :
                thisbar.stop_it() ## stops everything (no dismiss)
       
    def cancelAll(self):
        for worker in self.listofworkers :
            worker.cancel()
    
    def newProgressBar(self, newpbar):
        self.listofprogressbars.append(newpbar)
        switchboard.notify("new progressbar", newpbar)
            
        
        
        
    def NumThreadsInProgress(self):
        ## Calculates how many threads are running. 
        numthreadinprogress = 0
        for thisthread in self.listofprogressbars:
            if thisthread.get_success() == 0:
                numthreadinprogress +=1
        return numthreadinprogress

    def NumThreadsDone(self):
        ## Calculates how many threads exist in the list that are finished.
        numthreadsdone = 0
        for thisthread in self.listofprogressbars:
            if thisthread.get_success() != 0:
                numthreadsdone +=1
        return numthreadsdone

    def newWorker (self, newworker):
        ## print "adding worker"
        self.listofworkers.append(newworker)

    def removeWorker(self, oldworker):
        ## print "removing worker"
        try:
            self.listofworkers.remove(oldworker)
        except Exception:
            pass



threadManager = ThreadManager() 

##import atexit
##atexit.register(threadManager.quit) ## automatically stop any running thread that was left behind at exit.

