# -*- python -*-
# $RCSfile: debug.py,v $
# $Revision: 1.18 $
# $Author: langer $
# $Date: 2004/08/30 21:10:43 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import string
import sys, re
import types
import SWIG.common.lock
from SWIG.common.IO import threadstate
from common import parallel_enable

_debug_mode = 0
_debug_tag = ""

# The lock makes sure that threads don't intermix their debug lines.
lock = SWIG.common.lock.Lock()

##class DebugLock:
##    # A rather insecure lock for debugging purposes.
##    def acquire(self): pass
##    def release(self): pass
##lock = DebugLock()

def set_debug_mode():
    global _debug_mode
    _debug_mode = 1

def set_debug_tag(tag_string):
    global _debug_tag
    _debug_tag = tag_string
    
def clear_debug_mode():
    global _debug_mode
    global _debug_tag
    _debug_mode = 0
    _debug_tag = ""


def debug():
    return _debug_mode

import traceback

def dumpTrace(start=0, end=-1):
    if _debug_mode:
        lock.acquire()
        try:
            stack = traceback.extract_stack()
            depth = len(stack)
            thread = threadstate.findThreadNumber()
            if parallel_enable.query():
                from SWIG.common import mpitools
                rank="%02d" % mpitools.rank()
            else:
                rank='--'
            
            lines = ['+++-%04d-%02d-%s--------- debug.dumpTrace --------------'
                     % (depth, thread, rank)
                     ]
##            print >> sys.stderr, '+++----------- debug.dumpTrace --------------'
            for line in stack[start:end]:
                lines.append('+++%30s:%3d\t%s\t\t%s' % (line[0],line[1],
                                                        line[2],line[3]))
##                print >> sys.stderr, '+++%30s:%3d\t%s\t\t%s' % (line[0],line[1],
##                                                                line[2],line[3])
            lines.append('+++-------------- end trace -----------------')
##            print >> sys.stderr, '+++-------------- end trace -----------------'
            print >> sys.stderr, string.join(lines, '\n')
        finally:
            lock.release()


def msg(*args):
    if _debug_mode:
        lock.acquire()
        try:
            stack = traceback.extract_stack()
            depth = len(stack)
            thread = threadstate.findThreadNumber()
            if parallel_enable.query():
                from SWIG.common import mpitools
                rank="%02d" % mpitools.rank()
            else:
                rank='--'
            print >> sys.stderr, \
                  ('-%04d-%02d-%s'%(depth,thread,rank))+'-'*(depth-1), \
                  string.join(map(str, args), ' ')
        finally:
            lock.release()

def fmsg(*args):
    if _debug_mode:
        lock.acquire()
        try:
            stack = traceback.extract_stack()
            depth = len(stack)
            file = string.split(stack[-2][0], '/')[-1]
            func = stack[-2][2]
            line = stack[-2][1]
            thread = threadstate.findThreadNumber()
            if parallel_enable.query():
                from SWIG.common import mpitools
                rank="%02d" % mpitools.rank()
            else:
                rank='--'
            print >> sys.stderr, \
                  ('-%04d-%02d-%s'%(depth,thread,rank))+'-'*(depth-1), \
                  '%s(%d):%s'%(file, line, func),\
                  string.join(map(str, args), ' ')
        finally:
            lock.release()

# Labelled fmsg -- allows you to be picky about what to print.
def lfmsg(level, tag, *args):
    if _debug_mode >= level:
        lock.acquire()
        try:
            try:
                _debug_tag.index(tag)
            except ValueError:
                pass
            else:
                print >> sys.stderr, "%04d : %s : " % (level, tag),
                apply(fmsg, args)
        finally:
            lock.release()
    
    
from common import mainthread
import os

def mainthreadTest():
    if _debug_mode:
        if not mainthread.mainthread():
            fmsg("NOT IN MAIN THREAD")
            dumpTrace()
            os.abort()
