# -*- python -*-
# $RCSfile: strfunction.py,v $
# $Revision: 1.12 $
# $Author: reida $
# $Date: 2004/09/15 22:02:43 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import utils
from common.IO import parameter
import types

class StrFunction:
    def __init__(self, arglist, funcstr):
        self.arglist = arglist
        self.funcstr = funcstr
        if funcstr:
            self.function = utils.OOFeval('lambda %s: %s' %
                                          (self.arglist, self.funcstr))
        else:
            self.function = None
    def __call__(self, *args):
        if self.function:
            return self.function(*args)
    def string(self):
        return self.funcstr
    def __repr__(self):
        return "StrFunction('%s', '%s')" % (self.arglist, self.funcstr)
    def __str__(self):
        return "'%s'" % self.funcstr
    def __eq__(self,other):
        if other:
            if isinstance(other,StrFunction):
                if self.__class__==other.__class__:
                    if self.funcstr==other.funcstr:
                        return 1
        return 0

class XYStrFunction(StrFunction):
    def __init__(self, funcstr):
        StrFunction.__init__(self, 'x, y', funcstr)
    def __repr__(self):
        return "XYStrFunction('%s')" % self.funcstr


utils.OOFdefine('XYStrFunction', XYStrFunction)
utils.OOFdefine('StrFunction', StrFunction)


class XYStrFunctionParameter(parameter.ParameterBase):
    def __init__(self, name, value=None, default='0.0', tip=None):
        parameter.ParameterBase.__init__(self, name, value, default, tip)
    def checker(self, x):
        if type(x) is types.StringType:
            return
        if isinstance(x, XYStrFunction):
            return
        parameter.raiseTypeError(type(x), 'string or XYStrFunction')
    def set(self, value):
        if type(value) is types.StringType:
            self.__dict__['value'] = XYStrFunction(value)
        elif isinstance(value, XYStrFunction):
            self.__dict__['value'] = value
        elif value is None:
            self.__dict__['value'] = None
        else:
            raise parameter.ParameterMismatch(
                'Got ' + `value` + ' for Parameter ' + self.name)
        self.timestamp.increment()
