# -*- python -*-
# $RCSfile: skeletonBoundaryPage.py,v $
# $Revision: 1.33 $
# $Author: reida $
# $Date: 2004/09/16 20:16:50 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import gtk
from common import debug
from common import guitop
from common import labeltree
from common.IO import whoville
from common.IO.GUI import chooser
from common.IO.GUI import whowidget
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from engine import boundarybuilder
from engine import skeletoncontext
from engine.IO import boundarymenu
from SWIG.common import switchboard
from common import utils

class SkeletonBoundaryPage(oofGUI.MainPage):
    def __init__(self):
        
        oofGUI.MainPage.__init__(self, name="Skeleton Boundaries",
                                 ordering = 150,
                                 tip = "Create and orient boundaries.")

        mainbox = gtk.GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        hbox = gtk.GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = gtk.GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        
        self.skelwidget = whowidget.WhoWidget(whoville.getClass('Skeleton'),
                                              callback=self.new_skeleton,
                                              scope=self)
        label = gtk.GtkLabel('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[0], expand=0, fill=0)
        label = gtk.GtkLabel('Skeleton=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[1], expand=0, fill=0)

        mainpane = gtk.GtkHPaned()
        mainbox.add(mainpane)

##         boundaryopsframe = gtk.GtkFrame()
##         mainbox.add(boundaryopsframe)

##         boundaryopsbox = gtk.GtkHBox()
##         boundaryopsframe.add(boundaryopsbox)
        
        boundarylistframe = gtk.GtkFrame("Boundaries")
        mainpane.pack1(boundarylistframe, resize=0, shrink=0)
        
##         boundarylistframe = gtk.GtkFrame("Boundaries")
##         boundaryopsbox.add(boundarylistframe)

        boundarylistbox = gtk.GtkVBox()
        boundarylistframe.add(boundarylistbox)

        # List of all the boundaries.
        self.boundarylist = chooser.ChooserListWidget(
            callback = self.boundarylistCB)
        boundarylistbox.add(self.boundarylist.gtk, expand=1, fill=1)

        boundarybuttonbox = gtk.GtkHBox()
        boundarylistbox.add(boundarybuttonbox, expand=0, fill=0)

        # Buttons that actually do stuff.
        self.newbutton = gtk.GtkButton("New...")
        self.newbutton.connect("clicked", self.newBoundaryCB)
        guitop.top().tooltip(self.newbutton,
                             'Create a new skeleton boundary.')
        boundarybuttonbox.add(self.newbutton)

        self.editbutton = gtk.GtkButton("Modify...")
        self.editbutton.connect("clicked", self.modifyBoundaryCB)
        guitop.top().tooltip(self.editbutton,
                             'Modify the selected skeleton boundary.')
        boundarybuttonbox.add(self.editbutton)
        
        self.renamebutton = gtk.GtkButton("Rename...")
        self.renamebutton.connect("clicked", self.renameBoundaryCB)
        guitop.top().tooltip(self.renamebutton,
                             'Rename the selected skeleton boundary.')
        boundarybuttonbox.add(self.renamebutton)

        self.deletebutton = gtk.GtkButton("Delete")
        self.deletebutton.connect("clicked", self.deleteBoundaryCB)
        guitop.top().tooltip(self.deletebutton,
                             'Remove the selected skeleton boundary.')
        boundarybuttonbox.add(self.deletebutton)
        
        # TODO LATER: Copying could be added here -- the scenario is
        # that a user may want to make a copy of a boundary, and then
        # edit one of the copies.  Currently boundary editing is
        # primitive (one can only add/remove components), but when
        # visual pointy-clicky boundary editing is added, copying will
        # make sense.

        infoframe = gtk.GtkFrame("Boundary data")
        mainpane.pack2(infoframe, resize=1, shrink=1)

        infowindow = gtk.GtkScrolledWindow()
        infoframe.add(infowindow)
        infowindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        
        self.infotext = gtk.GtkText()
        self.infotext.set_word_wrap(1)
        infowindow.add_with_viewport(self.infotext)
        
        switchboard.requestCallback(("new who", 'Skeleton'),
                                    self.new_skeleton)
        
        # Catches push events *after* the boundaries have been
        # propagated, and also undo/redo events.  "who changed" is
        # too early.
        switchboard.requestCallback("new boundary configuration",
                                    self.newBdyConfigCB)
        switchboard.requestCallback("new boundary created",
                                    self.newBdyCB)
        switchboard.requestCallback("boundary removed",
                                    self.newBdyCB)
        switchboard.requestCallback("boundary renamed",
                                    self.newBdyCB)
        
        self.skelctxt = None
        self.sensitize()
        
    def new_skeleton(self, skelname):
        if labeltree.makePath(skelname) != \
               labeltree.makePath(self.skelwidget.get_value()):
            self.skelwidget.set_value(skelname)
        self.skelctxt = skeletoncontext.skeletonContexts[
            self.skelwidget.get_value()]
        self.sensitize()
        
    def sensitize(self, select=None, autoselect=1):
        if self.skelctxt:
            self.boundarylist.update(self.skelctxt.allBoundaryNames())
        else:
            self.boundarylist.update([])
        if select is not None:
            self.boundarylist.set_selection(select)
        # Deselecting the singleton autoselection, when a skeleton is being
        # loaded from a file - we don't want NO selection in the page.
        if not autoselect and self.boundarylist.has_selection():
            self.boundarylist.current_obj = None
        self.boundary_sensitize()

    def boundary_sensitize(self):
        buttons_alive = 0
        if self.boundarylist.has_selection():
            buttons_alive = 1
        self.editbutton.set_sensitive(buttons_alive)
        self.renamebutton.set_sensitive(buttons_alive)
        self.deletebutton.set_sensitive(buttons_alive)

        # "New" button done separately.
        if self.skelctxt:
            self.newbutton.set_sensitive(1)
        else:
            self.newbutton.set_sensitive(0)
            
        self.infotext.backward_delete(self.infotext.get_length())
        bdyname = self.boundarylist.get_value()
        
        if bdyname is not None:
            bdytext = self.skelctxt.boundaryInfo(bdyname)
            self.infotext.insert_defaults("Boundary %s:\n" % bdyname)
            self.infotext.insert_defaults(bdytext)
        else:
            self.infotext.insert_defaults("No boundary selected.")

        if self.skelctxt:
            self.skelctxt.bdytimestamp.increment()
            switchboard.notify("boundary selected", self.skelctxt, bdyname)
            switchboard.notify("redraw")

    def newBdyConfigCB(self, skeletoncontext):
        if skeletoncontext==self.skelctxt:
            self.sensitize()

    def newBdyCB(self, skeletoncontext, name, autoselect):
        if skeletoncontext==self.skelctxt:
            if autoselect:
                self.sensitize(select=name)
            else:
                self.sensitize(autoselect=autoselect)

    def bdyRemovedCB(self, skeletoncontext, name):
        if skeletoncontext==self.skelctxt:
            self.sensitize()        

    def bdyRenamedCB(self, skeletoncontext, name):
        if skeletoncontext==self.skelctxt:
            self.sensitize(select=name)        

    def whoChangedCB(self, skeletoncontext):
        if skeletoncontext==self.skelctxt:
            self.boundary_sensitize()
        
    def boundarylistCB(self, bdyobj):
        self.boundary_sensitize()

    # Button callback.
    # TODO LATER: The boundary creation process is not very
    # context-sensitive -- it's possible to specify the wrong kind of
    # "Director" (e.g. "Clockwise" for a segment set that is not a
    # loop, etc.), because the "sequencer" function that detects loops
    # does not get run until after the menu command has already
    # started.  This means that nonsensical operations generate
    # errors, instead of being disallowed.  Boundary creation
    # semantics should be rearranged, so that the first thing to do is
    # pick the "source object" (segment, element, or node group, or
    # selection).  The sequencer could be run at that time, and then
    # only sensible boundary presented as choices.  This prevents the
    # sequencer from being run too often, while still providing
    # context-sensitivity.
    def newBoundaryCB(self, gtkobj):
        menuitem = boundarymenu.boundarymenu.Construct
        nameparam =  menuitem.get_arg('name')
        builderparam = menuitem.get_arg('constructor')
        if parameterwidgets.getParameters(
            nameparam, builderparam, title="New Boundary",scope=self):
            menuitem.callWithDefaults(skeleton=self.skelwidget.get_value())
    
    def modifyBoundaryCB(self, gtkobj):
        # The parameter widget needs to know what types of modifiers to
        # present -- this can be passed through via the "parameter",
        # which should have it's ".modifiertype" attribute set to the
        # same as the current boundary object.
        #
        # TODO LATER: Can "modifiertype" be passed as a kwarg to
        # getParameters, and find its way through the makeWidget call?

        menuitem = boundarymenu.boundarymenu.Modify
        modparam = menuitem.get_arg('modifier')

        skelctxt = skeletoncontext.skeletonContexts[
            self.skelwidget.get_value()]
        bdyobj = skelctxt.getBoundary(self.boundarylist.get_value())

        modparam.modifiertype = bdyobj.modifiertype

        if parameterwidgets.getParameters(
            modparam, title="Boundary modifier",scope=self):
            menuitem.callWithDefaults(skeleton=self.skelwidget.get_value(),
                                      boundary=self.boundarylist.get_value())

    def renameBoundaryCB(self, gtkobj):
        menuitem = boundarymenu.boundarymenu.Rename
        newname = menuitem.get_arg('name')
        newname.set(self.boundarylist.get_value())
        if parameterwidgets.getParameters(
            newname, title="New name for this boundary"):
            menuitem.callWithDefaults(
                skeleton=self.skelwidget.get_value(),
                boundary=self.boundarylist.get_value())

    def deleteBoundaryCB(self, gtkobj):
        menuitem = boundarymenu.boundarymenu.Delete
        menuitem(skeleton=self.skelwidget.get_value(),
                 boundary=self.boundarylist.get_value())

        
    
sbp = SkeletonBoundaryPage()
