# -*- python -*-
# $RCSfile: skeletonbdytoolboxGUI.py,v $
# $Revision: 1.9 $
# $Author: langer $
# $Date: 2004/01/10 16:33:16 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# GUI part of the SkeletonBoundaryToolbox.

from engine.IO import skeletonbdytoolbox
from common.IO.GUI import toolboxGUI
from common.IO.GUI import mousehandler
from SWIG.common import switchboard
import gtk

from common.IO.GUI import chooser


class SkeletonBoundaryToolboxGUI(toolboxGUI.GfxToolbox,
                                 mousehandler.MouseHandler):
    def __init__(self, bdytoolbox):
        debug.mainthreadTest()
        toolboxGUI.GfxToolbox.__init__(self, "Skeleton Boundaries", bdytoolbox)
        self.edge_picker = chooser.MultiListWidget([], [], self.newPick)
        self.point_picker = chooser.MultiListWidget([], [], self.newPick)
        self.newpicker = 1
        vbox = gtk.GtkVBox()
        self.gtk.add(vbox)

        eframe = gtk.GtkFrame("Edge boundaries:")
        vbox.add(eframe)
        scroll = gtk.GtkScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        eframe.add(scroll)
        scroll.add_with_viewport(self.edge_picker.gtk)

        pframe = gtk.GtkFrame("Point boundaries:")
        vbox.add(pframe)
        scroll = gtk.GtkScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        pframe.add(scroll)
        scroll.add_with_viewport(self.point_picker.gtk)

        # Need to get new boundary sets even if we aren't active,
        # so this callback goes here.
        switchboard.requestCallback((self.gfxwindow(),
                                     'new boundary set'), self.newBoundarySet)

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.activecallbacks = []
            # layerChangeCB is in the parent.
            # switchboard.requestCallback((self.gfxwindow(),
            #                              'layers changed'),
            #                             self.layerChangeCB),
            # switchboard.requestCallback((self.gfxwindow(),
            #                              'new node selection'),
            #                             self.newSelection)

        # Selection state only needs to be set once, when the
        # widget first appears.  It can't be set in the __init__
        # to this structure, because the widget hasn't been updated
        # first.  So, on the first (and only the first) activation,
        # set the selection state from the gfxwindow setting.
        #   NB the gfxwindow setting is a WhatList object, defined in
        # engine/IO/skeletonboundaryext.py
        if self.newpicker:
            sel_set = self.gfxwindow().settings.boundaryNames
            self.edge_picker.set_selection(sel_set.edges)
            self.point_picker.set_selection(sel_set.points)
            self.newpicker = 0
            
    def deactivate(self):
        toolboxGUI.GfxToolbox.deactivate(self)
        map(switchboard.removeCallback, self.activecallbacks)

    # Argument is discarded because we don't know/care which
    # widget it came from.
    def newPick(self, selectlist):
        switchboard.notify((self.gfxwindow(), "new boundary pick"),
                           (self.edge_picker.get_value(),
                            self.point_picker.get_value() ) )

    # Called-back when the layers change, meaning that there could be
    # a new set of boundaries from which to pick -- inform the widget.
    def newBoundarySet(self, boundaryset):
        self.edge_picker.update(boundaryset[0])
        self.point_picker.update(boundaryset[1])

def _bdy_makeGUI(self):
    return SkeletonBoundaryToolboxGUI(self)

skeletonbdytoolbox.SkeletonBoundaryToolbox.makeGUI = _bdy_makeGUI
