# -*- python -*-
# $RCSfile: outputDefs.py,v $
# $Revision: 1.50 $
# $Author: langer $
# $Date: 2004/06/17 21:10:31 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# This file generates concrete Outputs from the Outputs defined in
# outputClones.py.  The concrete Outputs are stored in a tree
# structure which can easily be converted to a menu.

from SWIG.common import switchboard
from SWIG.engine import field
from SWIG.engine import flux
from SWIG.engine import outputval
from SWIG.engine import planarity
from SWIG.engine.IO import propertyoutput
from common import debug
from common import enum
from common.IO import output
from common.IO import parameter
from common import primitives
from engine import problem
from engine.IO import outputClones


from types import *
import string

######################################

# Displacement

# Although Displacement is a Field, it's convenient to treat it as a
# Point so that it can be added to a position.  Defining
# DisplacementOutput in terms of another output which sets a parameter
# to problem.Displacement allows the computability checking to work.

Displacement = field.getCompoundField("Displacement")
iter = Displacement.iterator(planarity.IN_PLANE)
disp0 = iter.cloneIndex()
iter.next()
disp1 = iter.cloneIndex()

def _disp2point(fieldlist, *args, **kwargs):
    # Convert displacement field OutputVals to Points
    return [primitives.Point(field[disp0], field[disp1]) for field in fieldlist]

displacementFieldOutput = outputClones.FieldOutput.clone(
    name="displacement field")
displacementFieldOutput.plugParameter(which="field", value=Displacement)
displacementOutput = output.Output(name="dispPoint",
                                   oType=primitives.Point,
                                   func=_disp2point,
                                   inputs=output.Input("displacement",
                                                       outputval.OutputValPtr,
                                                       displacementFieldOutput))

##def _list2point(list, *args, **kwargs):
##    return [primitives.Point(p[0], p[1]) for p in list]

##List2PointOutput = \
##                 output.Output(name="list2point", func=_list2point,
##                               oType=primitives.Point,
##                               inputs=(output.Input("vector",
##                                            parameter.IndexableTypeChecker()),))

##displacementFieldOutput = \
##                outputClones.VectorFieldOutput.clone(name="displacement field")
##displacementFieldOutput.plugParameter(which='field', value=Displacement)

##displacementOutput = List2PointOutput.clone(name="displacement")
##displacementOutput.plugInput(input=displacementFieldOutput, which="vector")


# Enhanced position is scalefactor*Displacement + original position
enhancedPosition = outputClones.PointSumOutput.clone(name='enhanced position')
enhancedPosition.plugInput(displacementOutput, 'point1')
enhancedPosition.plugInput(outputClones.posOutput, 'point2')
enhancedPosition.plugParameter(value=1.0, which='b')
enhancedPosition.plugParameter(default=1.0, which='a', rename='factor')

# Actual position is enhanced position with a scalefactor of 1
actualPosition = enhancedPosition.clone(name='actual position')
actualPosition.plugParameter(value=1.0, which='factor')

originalPosition = outputClones.posOutput
output.definePositionOutput('original', outputClones.posOutput)
output.definePositionOutput('actual', actualPosition)
output.definePositionOutput('enhanced', enhancedPosition)

######################

output.defineScalarOutput('Field:Component', outputClones.FieldCompOutput)
output.defineScalarOutput('Field:Invariant', outputClones.FieldInvariantOutput)
output.defineScalarOutput('Field:Derivative:Component',
                          outputClones.FieldDerivCompOutput)
output.defineScalarOutput('Field:Derivative:Invariant',
                          outputClones.FieldDerivInvariantOutput)
output.defineScalarOutput('Flux:Component', outputClones.FluxCompOutput)
output.defineScalarOutput('Flux:Invariant', outputClones.FluxInvariantOutput)

from common import strfunction
xyfunc = outputClones.FunctionOutput.clone()
xyfunc.plugParameter(default=strfunction.XYStrFunction('1.0'), which='f')
output.defineScalarOutput('XYFunction', xyfunc)

######################

# PropertyOutputs, which are computed by the Property objects in a
# Material, only have to be identified by name, type, and (possibly)
# initializer here.  The computations are all done by the
# PropertyOutput<TYPE> classes and the various Property::output virtual
# functions.

class EnergyType(enum.EnumClass("Total", "Elastic")): pass

propertyoutput.ScalarPropertyOutputRegistration(
    "Energy",
    parameters=enum.EnumParameter("etype", EnergyType, default="Total"))

