# -*- python -*-
# $RCSfile: propertyoutput_.py,v $
# $Revision: 1.9 $
# $Author: langer $
# $Date: 2004/06/04 20:57:29 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from SWIG.engine import outputval
from SWIG.engine import symmmatrix
from common import debug
from common.IO import output
from engine.IO import outputClones
import types

# The PropertyOutputRegistration subclasses create an Output object
# for each registered PropertyOutput.  This bridges the gap between
# the C++ PropertyOutputs and the more general Python Outputs.


##### Scalar outputs

class ScalarPropertyOutputRegistration(PropertyOutputRegistration):
    def __init__(self, name, initializer=None, parameters=[]):
        PropertyOutputRegistration.__init__(
            self, name, initializer or ScalarPropertyOutputInit())
        op = output.Output(name=name, func=self.opfunc,
                           oType=types.FloatType,
                           parameters=parameters)
        output.defineScalarOutput(name, op)

    def opfunc(self, mesh, elements, coords, parameters, *args, **kwargs):
        # Returns a list of Floats.
        ans = []
        po = self.instantiate([p.value for p in parameters])
        initializer = self.initializer()
        for element, ecoords in zip(elements, coords):
            ans += po.evaluate(mesh, element, initializer, ecoords)
        return [a.value() for a in ans] # convert from ScalarOutputVal to Float
    
##### SymmMatrix3 outputs.

class SymmMatrix3PropertyOutputRegistration(PropertyOutputRegistration):
    def __init__(self, name, initializer=None, parameters=[]):
        PropertyOutputRegistration.__init__(
            self, name, initializer or SymmMatrix3PropertyOutputInit())
        op = output.Output(name=name, func=self.opfunc,
                           oType=outputval.OutputValPtr,
                           parameters=parameters)

        compout = outputClones.ComponentOutput.clone(
            name=name+" Component",
            inputs=output.Input("field", outputval.OutputValPtr,
                                op.clone()))
        output.defineScalarOutput(name+":Component", compout)
        
        invout = outputClones.InvariantOutput.clone(
            name=name+" Invariant",
            inputs=output.Input("field", outputval.OutputValPtr,
                                op.clone()))
        output.defineScalarOutput(name+":Invariant", invout)

    def opfunc(self, mesh, elements, coords, parameters, *args, **kwargs):
        # Returns a list of OutputVal instances.
        ans = []
        po = self.instantiate([p.value for p in parameters])
        initializer = self.initializer()
        for element, ecoords in zip(elements, coords):
            ans += po.evaluate(mesh, element, initializer, ecoords)
        return ans
