# -*- python -*-
# $RCSfile: skeletonselectiontoolbox.py,v $
# $Revision: 1.14 $
# $Author: langer $
# $Date: 2004/09/21 19:23:57 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import debug
from common import labeltree
from common.IO import genericselecttoolbox
from common import toolbox
from common.IO import whoville
from engine import skeletonselmodebase

## There are three skeleton selection toolboxes (so far), for
## selecting Elements, Nodes, and Segments.  Each is a subclass of
## SkeletonSelectionToolbox.  The subclasses are created automatically
## by instancing a subclass of SkeletonSelectionMode.


#############################

class SkeletonSelectionToolbox(genericselecttoolbox.GenericSelectToolbox):
    def __init__(self, mode, gfxwindow):
        ## 'mode' is a SkeletonSelectionMode object.  It's stored in
        ## the extrakwargs dict in the GenericSelectToolbox, and is
        ## passed from GenericSelectToolbox.getSelection() to
        ## SkeletonContext.getSelectionContext(), where it's used to
        ## retrieve the appropriate Selection object from the
        ## SkeletonContext.
        self.mode = mode
        genericselecttoolbox.GenericSelectToolbox.__init__(
            self,
            name="Select_"+self.modename(),
            gfxwindow=gfxwindow,
            mode=mode,
            method=mode.methodclass)
    def modename(self):
        return self.mode.name
    def emptyMessage(self):
        return "No Skeleton!"           # you spineless bastard

    def sourceParams(self):
        return [whoville.WhoParameter('skeleton',
                                      whoville.getClass('Skeleton'))]

    def setSourceParams(self, menuitem, source):
        menuitem.get_arg('skeleton').value = source.path()

    def getSourceObject(self, params, gfxwindow):
        ## See comments in common/IO/pixelselectiontoolbox.py.  We're
        ## expecting to find a Skeleton or Mesh, but always return the
        ## Skeleton.
        whopath = labeltree.makePath(params['skeleton'])
        return whoville.getClass('Skeleton')[whopath[:2]]

    def signal(self, method, pointlist):
        # Called by GenericSelectToolbox.selectCB after performing a
        # selection.
        switchboard.notify(self.mode.newselectionsignal, method, pointlist)
        switchboard.notify(self.mode.changedselectionsignal)
        switchboard.notify("redraw")

## Create toolbox subclasses for each SkeletonSelectionMode

tbordering=2.5

# In Python 2.2 and later, do it this way...
# def newSelectionMode(mode):
#     class SkelSelectToolbox(SkeletonSelectionToolbox):
#         def __init__(self, gfxwindow):
#             SkeletonSelectionBoolbox.__init__(self, mode=mode,
#                                               gfxwindow=gfxwindow)
#      ... etc ...

# This global-variable machinery here is so that each new
# SkelSelectToolbox class has its own mode argument, not shared with
# other, similar classes.  It's complicated by the fact that, in
# Python 2.1, the class definition has its own scope, and cannot see
# variables in the enclosing function scope.  The Python 2.2 and later
# way to do this is in the comment above.
def _newSelectionMode(mode):
    global global_mode_memory_variable
    global_mode_memory_variable = mode
    class SkelSelectToolbox(SkeletonSelectionToolbox):
        mode = global_mode_memory_variable
        def __init__(self, gfxwindow):
            SkeletonSelectionToolbox.__init__(self,
                                              # mode is extrakwargs.
                                              mode=self.__class__.mode,
                                              gfxwindow=gfxwindow)
        global tbordering
        tbordering += 0.01

    ## The mode keeps a reference to the associated class so that GUI
    ## extensions can be applied to the class.  If the mode didn't
    ## keep a reference, then the only way to find the class would be
    ## to look through the registry in toolbox.py.
    mode.tbclass = SkelSelectToolbox
    toolbox.registerToolboxClass(SkelSelectToolbox, ordering=tbordering)
    
for mode in skeletonselmodebase.SkeletonSelectionMode.modes:
    _newSelectionMode(mode)
    
switchboard.requestCallback(skeletonselmodebase.SkeletonSelectionMode,
                            _newSelectionMode)
