# -*- python -*-
# $RCSfile: skeletonselectmenu.py,v $
# $Revision: 1.12 $
# $Author: reida $
# $Date: 2004/03/05 20:56:45 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# The menu for node-selection modification operations, automatically
# added via switchboard callback.

from SWIG.common import lock
from SWIG.common import switchboard
from engine import skeletonselectionmod
from engine import skeletoncontext
from common.microstructure import getMicrostructure
from common import utils
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import parameter
from common.IO import whoville
import types

topmenu = mainmenu.OOF

nodeselectmenu = oofmenu.OOFMenuItem('NodeSelection', cli_only=1)
topmenu.addItem(nodeselectmenu)

segmentselectmenu = oofmenu.OOFMenuItem('SegmentSelection', cli_only=1)
topmenu.addItem(segmentselectmenu)

elementselectmenu = oofmenu.OOFMenuItem('ElementSelection', cli_only=1)
topmenu.addItem(elementselectmenu)

def _undo(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.undo()
    finally:
        selection.end_writing()
    selection.signal()

def _redo(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.redo()
    finally:
        selection.end_writing()
    selection.signal()

def _clear(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.start() # Clear should be undoable.
        selection.clear()
    finally:
        selection.end_writing()
    selection.signal()

def _invert(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.start() # Invert should be undoable.
        selection.invert()
    finally:
        selection.end_writing()
    selection.signal()



def makeMenu(menu, modifier, selection_name):
    menu.clearMenu()

    undo_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Undo", callback=_undo,
        params = [ whoville.WhoParameter("skeleton",
                                         whoville.getClass('Skeleton') )]))
    undo_item.data = selection_name

    redo_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Redo", callback=_redo,
              params = [ whoville.WhoParameter("skeleton",
                                         whoville.getClass('Skeleton') )]))
    redo_item.data = selection_name

    clear_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Clear", callback=_clear,
        params = [ whoville.WhoParameter("skeleton",
                                         whoville.getClass('Skeleton') )]))
    clear_item.data = selection_name

    invert_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Invert", callback=_invert,
        params = [ whoville.WhoParameter("skeleton",
                                         whoville.getClass('Skeleton') )]))
    invert_item.data = selection_name    
        
    for r in modifier.registry:
        menuitem = menu.addItem(
            oofmenu.OOFMenuItem(utils.space2underscore(r.name()),
                                callback = skeletonselectionmod.modify,
                                params = [
            whoville.WhoParameter("skeleton",
                                  whoville.getClass('Skeleton') ) ] +
                                r.params))
        menuitem.data = r

# Callback for new registration entries added in later imports.
switchboard.requestCallback(skeletonselectionmod.NodeSelectionModifier,
                            makeMenu, nodeselectmenu,
                            skeletonselectionmod.NodeSelectionModifier,
                            "nodeselection")

switchboard.requestCallback(skeletonselectionmod.SegmentSelectionModifier,
                            makeMenu, segmentselectmenu,
                            skeletonselectionmod.SegmentSelectionModifier,
                            "segmentselection")

switchboard.requestCallback(skeletonselectionmod.ElementSelectionModifier,
                            makeMenu, elementselectmenu,
                            skeletonselectionmod.ElementSelectionModifier,
                            "elementselection")


# Insert menu items for the ones already in the registry at
# import-time.
makeMenu(nodeselectmenu, skeletonselectionmod.NodeSelectionModifier,
         "nodeselection")

makeMenu(segmentselectmenu,
         skeletonselectionmod.SegmentSelectionModifier,
         "segmentselection")

makeMenu(elementselectmenu,
         skeletonselectionmod.ElementSelectionModifier,
         "elementselection")
