# -*- python -*-
# $RCSfile: boundtest.py,v $
# $Revision: 1.5 $
# $Author: reida $
# $Date: 2003/04/02 14:52:05 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from skeleton import ghost_coord

class TwoNodeMove:
    """
    If an element boundary looks like this(1 & 0 are pixel colors);

              3     edge 2   2
               11111111111111
               1            1
        edge 3 1            1 edge 1
               0            0
               00000000000000 
              0     edge 0   1

    point1 : in-between point of pixels 0 and 1 on edge 1
    point2 : in-between point of pixels 1 and 0 on edge 3
    side1  : edge 1
    side2  : edge 3

    In this setting, if two node(corners) pairs (either 0-1 or 2-3) move to
    point1 and point2 respectively, the move may enhances element's
    homogeneity up to the required level.
    The same goes for triangles.
    """
    def __init__(self,element,side1,side2,point1,point2):
        self.element=element
        self.sides=[side1,side2]
        self.points=[point1,point2]

class OneNodeMove:
    """
    In case like this;

              3     edge 2   2
               11111111111111
               1            1
        edge 3 1            1 edge 1
               1            0
               00000000000000
              0     edge 0   1
 
        point : in-between point of pixels 0 and 1 on edge 1
        side  : edge 1

    As in TwoNodeMove, if either node 1 or 2 moves to the point,
    the move may enhances element's homogeneity up to the required level.
    The same goes for triangles.
    """
    def __init__(self,element,side,point):
        self.element=element
        self.side=side
        self.point=[point]

class BoundaryCheck:
    def __init__(self):
        self.insidePixels=None
        self.edgePixels=None
        self.cornerIndices=None 
        self.BPoints=None
        self.quad2ND=None
        self.quad1ND=None
        self.tri2ND=None
        self.tri1ND=None

    def doBoundaryCheck(self,mesh,freeMove):
        self.quad2ND=[]
        self.quad1ND=[]
        self.tri2ND=[]
        self.tri1ND=[]
        for el in mesh.elements:
            if el.material==None:
                if self.checkBoundary(mesh,el,freeMove):
                    self.newAddition(el)

    def checkBoundary(self,mesh,el,freeMove):
        x=[c.position.x for c in el.corners]
        y=[c.position.y for c in el.corners]
        self.getInsidePixels(mesh,el,x,y)
        self.getCornerIndices(mesh,el,x,y)
        self.getEdgePixels(el,x,y)
        self.BPoints=[]
        for n in range(len(el.corners)):
            if len(self.edgePixels[n])==0:
                self.BPoints.append(ghost_coord(None,None))
                continue
            else:
                count=0
                previous=self.edgePixels[n][0].color
                for i in range(1,len(self.edgePixels[n])):
                    current=self.edgePixels[n][i].color
                    if count==0 and previous!=current:
                        if freeMove:
                            if not self.doFreeMove(mesh,el,n,i):
                                count=count+1
                                bpoint=self.bpointSet(mesh,n,i)
                        else:
                            count=count+1
                            bpoint=self.bpointSet(mesh,n,i)
                    elif count==1 and previous!=current:
                        if freeMove:
                            if not self.doFreeMove(mesh,el,n,i): return 0
                        else: return 0
                    previous=current
                if count==0: self.BPoints.append(ghost_coord(None,None))
                else: self.BPoints.append(bpoint)
        return 1

    def getInsidePixels(self,mesh,el,x,y):
        i1,i2,j1,j2=el.getBoundingBox(mesh,x,y)
        self.insidePixels=[[None for i in range(i1,i2+1)]\
                           for j in range(j1,j2+1)]
        if len(el.corners)==3: x.append(x[0]); y.append(y[0])
        for n in range(j1,j2+1):
            for m in range(i1,i2+1):
                xx=mesh.pixels[n][m].position.x
                yy=mesh.pixels[n][m].position.y
                if el.insideQuad(x,y,xx,yy):
                    self.insidePixels[n-j1][m-i1]=mesh.pixels[n][m]
        if len(el.corners)==3: del x[3]; del y[3]

    def getCornerIndices(self,mesh,el,x,y):
        i1,i2,j1,j2=el.getBoundingBox(mesh,x,y)
        self.cornerIndices=[[None,None] for i in range(len(el.corners))]
        for i in range(len(el.corners)):
            x0=x[i]; y0=y[i]
            for n in range(j1,j2+1):
                for m in range(i1,i2+1):
                    if self.insidePixels[n-j1][m-i1]!=None:
                        xx=mesh.pixels[n][m].position.x
                        yy=mesh.pixels[n][m].position.y
                        min=(xx-x0)**2+(yy-y0)**2; break
            for n in range(j1,j2+1):
                for m in range(i1,i2+1):
                    if self.insidePixels[n-j1][m-i1]!=None:
                        xx=mesh.pixels[n][m].position.x
                        yy=mesh.pixels[n][m].position.y
                        dist=(xx-x0)**2+(yy-y0)**2
                        if dist<=min:
                            self.cornerIndices[i][0]=n-j1
                            self.cornerIndices[i][1]=m-i1
                            min=dist

    def getEdgePixels(self,el,x,y):
        self.edgePixels=[]
        div=len(el.corners)
        for n in range(len(el.corners)):
            case,start,end,direction=self.setupEdgePixelSearch(el,n)
            temp=[]
            if direction==1:
                temp.append(self.insidePixels[self.cornerIndices[n%div][0]]\
                                         [self.cornerIndices[n%div][1]])
            else:
                temp.append(self.insidePixels[self.cornerIndices[(n+1)%div][0]]\
                                         [self.cornerIndices[(n+1)%div][1]])
            for m in range(start,end+1):
                if self.getThePixel(el,case,n,m)!=None:
                    temp.append(self.getThePixel(el,case,n,m))
            if direction==1:
                temp.append(self.insidePixels[self.cornerIndices[(n+1)%div][0]]\
                                          [self.cornerIndices[(n+1)%div][1]])
            else:
                temp.append(self.insidePixels[self.cornerIndices[n%div][0]]\
                                          [self.cornerIndices[n%div][1]])
            if direction==-1: temp.reverse()
            self.edgePixels.append(temp)

    def setupEdgePixelSearch(self,el,n):
        div=len(el.corners)
        n1=el.corners[n%div]; n2=el.corners[(n+1)%div]
        dX=abs(n2.position.x-n1.position.x)
        dY=abs(n2.position.y-n1.position.y)
        if -dX<=dY<=dX: case=1
        else: case=0
        if self.cornerIndices[n%div][case]<=self.cornerIndices[(n+1)%div][case]:
            start=self.cornerIndices[n%div][case]+1
            end=self.cornerIndices[(n+1)%div][case]-1
            direction=1
        else:
            start=self.cornerIndices[(n+1)%div][case]+1
            end=self.cornerIndices[n%div][case]-1
            direction=-1
        return case,start,end,direction
            
    def getThePixel(self,el,case,n,m):
        x1=el.corners[n%len(el.corners)].position.x
        y1=el.corners[n%len(el.corners)].position.y
        x2=el.corners[(n+1)%len(el.corners)].position.x
        y2=el.corners[(n+1)%len(el.corners)].position.y
        theOne=None
        if case==0:
            for i in range(len(self.insidePixels[0])):
                if self.insidePixels[m][i]!=None:
                    x=self.insidePixels[m][i].position.x
                    y=self.insidePixels[m][i].position.y
                    min=self.distance(x1,y1,x2,y2,x,y); break
            for i in range(len(self.insidePixels[0])):
                if self.insidePixels[m][i]==None: continue
                x=self.insidePixels[m][i].position.x
                y=self.insidePixels[m][i].position.y
                dist=self.distance(x1,y1,x2,y2,x,y)
                if dist<=min: 
                    theOne=i
                    min=dist
            if theOne==None: return None
            return self.insidePixels[m][theOne]
        else:
            for i in range(len(self.insidePixels)):
                if self.insidePixels[i][m]!=None:
                    x=self.insidePixels[i][m].position.x
                    y=self.insidePixels[i][m].position.y
                    min=self.distance(x1,y1,x2,y2,x,y); break
            for i in range(len(self.insidePixels)):
                if self.insidePixels[i][m]==None: continue
                x=self.insidePixels[i][m].position.x
                y=self.insidePixels[i][m].position.y
                dist=self.distance(x1,y1,x2,y2,x,y)
                if dist<=min:
                    theOne=i
                    min=dist
            if theOne==None: return None
            return self.insidePixels[theOne][m]

    def distance(self,x1,y1,x2,y2,r,s):
        if x1==x2 and y1==y2:
            print "Something wrong in distance"
        elif x1==x2:
            D=abs(x1-r)
        elif y1==y2:
            D=abs(y1-s)
        else:
            A=(y2-y1)/(x2-x1)
            B=-1.0
            C=y1-(y2-y1)/(x2-x1)*x1
            D=abs(A*r+B*s+C)/(A**2+B**2)**0.5
        return D

    def doFreeMove(self,mesh,el,n,i):
        corner=None
        if len(self.edgePixels[n])<=5: limit=1
        elif len(self.edgePixels[n])<=10: limit=2
        elif len(self.edgePixels[n])<=20: limit=3
        else: limit=5
        if i<=limit: corner=el.corners[n]
        if len(self.edgePixels[n])-i<=limit:
            corner=el.corners[(n+1)%len(el.corners)]
        if corner==None: return 0
        if corner.movability==12:
            X=corner.position.x; Y=corner.position.y
            corner.position=(self.edgePixels[n][i-1].position+\
                             self.edgePixels[n][i].position)*0.5
            if self.collapseCheck(el): return 0 
            self.adjustNeighbors(mesh,el,corner)
            return 1
        else: return 0

    def collapseCheck(self,el):
        x=[c.position.x for c in el.corners]
        y=[c.position.y for c in el.corners]
        div=len(el.corners)
        for i in range(len(el.corners)):
            if ((x[(i+1)%div]-x[i%div])**2+(y[(i+1)%div]-y[i%div])**2)==0.0:
                return 1
        return 0

    def adjustNeighbors(self,mesh,el,corner):
        neighbors=[]
        for e in mesh.elements:
             if corner in e.corners:
                 neighbors.append(e)
        for i in range(len(neighbors)):
            e=neighbors[i]
            e.theTestRatio(mesh)

    def bpointSet(self,mesh,n,i):
        bpoint=ghost_coord(None,None)
        mX=mesh.MS._size[0]/(len(mesh.pixels[0])*1.0)
        mY=mesh.MS._size[1]/(len(mesh.pixels)*1.0)
        bpoint=(self.edgePixels[n][i-1].position+\
                self.edgePixels[n][i].position)*0.5
        if abs(bpoint.x)<(1.0*mX): bpoint.x=0.0
        if abs(bpoint.x-mesh.MS._size[0])<(1.0*mX): bpoint.x=mesh.MS._size[0]
        if abs(bpoint.y)<(1.0*mY): bpoint.y=0.0
        if abs(bpoint.y-mesh.MS._size[1])<(1.0*mY): bpoint.y=mesh.MS._size[1]
        return bpoint

    def newAddition(self,el):
        side=[]
        for i in range(len(el.corners)):
            if self.BPoints[i].x!=None and self.BPoints[i].y!=None:
                side.append(i)
        if len(el.corners)==4:
            if len(side)==2 and (side[0]+side[1])%2==0:
                tNM=TwoNodeMove(el,side[0],side[1],self.BPoints[side[0]],\
                    self.BPoints[side[1]])
                self.quad2ND.append(tNM)
            elif len(side)==1:
                oNM=OneNodeMove(el,side[0],self.BPoints[side[0]])
                self.quad1ND.append(oNM)
        elif len(el.corners)==3:
            if len(side)==2:
                tNM=TwoNodeMove(el,side[0],side[1],self.BPoints[side[0]],\
                    self.BPoints[side[1]])
                self.tri2ND.append(tNM)
            elif len(side)==1:
                oNM=OneNodeMove(el,side[0],self.BPoints[side[0]])
                self.tri1ND.append(oNM)
