# -*- python -*-
# $RCSfile: skeletonselmodebase.py,v $
# $Revision: 1.3 $
# $Author: langer $
# $Date: 2004/01/31 01:30:13 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import ooferror
from SWIG.common import switchboard

# Each category of thing in a Skeleton which can be selected by the
# user (such as Elements, Nodes, and Segments) must have a subclass of
# SkeletonSelectionMode, and a single instance of that subclass must
# be created at startup time.  The SkeletonSelectionMode subclass
# stores meta-data about the selectable objects, and is used by the
# SkeletonSelectionPage and SkeletonSelectionToolbox to automatically
# generate the user interface and switchboard signals required to
# select the objects.

class SkeletonSelectionMode:
    modes = []
    def __init__(self, name, methodclass, modifierclass, newselectionsignal,
                 modifierappliedsignal, changedselectionsignal, groupmenu):
        self.name = name

        # methodclass is the RegisteredClass of routines for creating
        # new selections.  These appear in the
        # SkeletonSelectionToolbox.
        self.methodclass = methodclass

        # modifierclass is the RegisteredClass of routines for
        # modifying existing selections.  These appear in the
        # SkeletonSelectionPage.
        self.modifierclass = modifierclass

        # newselectionsignal is sent along with a selection method and
        # pointlist when a new selection is made.
        # changedselectionsignal is sent when any change is made in
        # the selection at all.
        # modifierappliedsignal is sent along with a selection
        # modifier when the selection is modified.
        self.modifierappliedsignal = modifierappliedsignal
        self.newselectionsignal = newselectionsignal
        self.changedselectionsignal = changedselectionsignal

        # groupmenu is the menu of commands for manipulating groups of
        # the object.
        self.groupmenu = groupmenu

        SkeletonSelectionMode.modes.append(self)
        switchboard.notify(SkeletonSelectionMode, self)
    def toolboxName(self):
        return "Select_" + self.name
    def modifierApplied(self, modifier):
        switchboard.notify(self.modifierappliedsignal, modifier)
    def getSelectionContext(self, skeletoncontext):
        raise ooferror.ErrPyProgrammingError("SkeletonSelectionMode.getSelectionContext() needs to be redefined in class " + self.__class__.__name__)
    def getGroups(self, skeletoncontext):
        raise ooferror.ErrPyProgrammingError("SkeletonSelectionMode.getGroups() needs to be redefined in class " + self.__class__.__name__)
    def getSelectionMenu(self):
        raise ooferror.ErrPyProgrammingError("SkeletonSelectionMode.getSelectionMenu() needs to be redefined in class " + self.__class__.__name__)
    def getGroupMenu(self):
        raise ooferror.ErrPyProgrammingError("SkeletonSelectionMode.getGroupMenu() needs to be redefined in class " + self.__class__.__name__)


def getMode(name):
    for mode in SkeletonSelectionMode.modes:
        if mode.name == name:
            return mode


def firstMode():
    if SkeletonSelectionMode.modes:
        return SkeletonSelectionMode.modes[0]
