# -*- python -*-
# $RCSfile: smooth.py,v $
# $Revision: 1.21 $
# $Author: redwing $
# $Date: 2004/06/22 20:15:05 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import primitives
from common import registeredclass
from common.IO import reporter
from common.IO import parameter
from engine import fiddlenodesbase
from engine import skeletonmodifier
from SWIG.common import switchboard
from SWIG.common.IO import progressbar
import random
import types

FiddleNodes = fiddlenodesbase.FiddleNodes
FiddleNodesTargets = fiddlenodesbase.FiddleNodesTargets
IterationManager = fiddlenodesbase.IterationManager

class SmoothMovePosition(fiddlenodesbase.FiddleNodesMovePosition):
    def __call__(self, skeleton, node):
        avgPos = primitives.Point(0.,0.)
        nbrNodes = node.neighborNodes(skeleton)
        for n in nbrNodes:
            avgPos += n.position()
        avgPos /= len(nbrNodes)
        return avgPos

registeredclass.Registration(
    'Smooth Move Position', fiddlenodesbase.FiddleNodesMovePosition,
    SmoothMovePosition,
    ordering=1)

class Smooth(FiddleNodes, skeletonmodifier.SkeletonModifier):
    def __init__(self, targets, criterion, T, iteration):
        FiddleNodes.__init__(self, targets, criterion, T, iteration)
        self.intro = "preparing to smooth skeleton...      "
        self.header = "Smoothing skeleton: "
        self.outro = "Smoothing done: "
        self.movedPosition = SmoothMovePosition()

    ## def movedPosition(self, skeleton, node):
    ##    avgPos = primitives.Point(0.,0.)
    ##    nbrNodes = node.neighborNodes(skeleton)
    ##    for n in nbrNodes:
    ##        avgPos += n.position()
    ##    avgPos /= len(nbrNodes)
    ##    return avgPos

####################################################

registeredclass.Registration('Smooth',
                             skeletonmodifier.SkeletonModifier,
                             Smooth,
                             ordering=4,
                             params=[
    parameter.Parameter('targets', FiddleNodesTargets,
                        tip='Which nodes to fiddle.'),
    parameter.Parameter('criterion', skeletonmodifier.SkelModCriterion,
                        tip = 'Acceptance criterion'),
    parameter.FloatParameter('T', value=0.0, tip='Even though a move failed to meet the acceptance criterion, it can be accepted if T>0 and exp(-diffE/T) > random, where diffE represents energy difference and random() represents a number between 0 and 1.'),
    parameter.Parameter('iteration', IterationManager,
                        tip='Iteration methods.')
    ],
                             tip='Move nodes to the average position of their respective neighbors and accept ones that meet the acceptance criterion.')
